@@ -66,7 +66,7 @@ my $center_y = $YY / 2;
 my $factor   = 3;
 my $rpi      = ($F->{'fscreeninfo'}->{'id'} =~ /BCM270(8|9)/i) ? 1 : 0;
 
-$factor *= 3 if ($rpi);    # Raspberry PI is sloooooow
+# $factor *= 3 if ($rpi);    # Raspberry PI is sloooooow
 my $BW = 0;
 foreach my $file (@files) {
     next if ($file =~ /^\.+/ || $file =~ /Test|gif/ || -d "$images_path/$file");
@@ -225,9 +225,9 @@ sub color_mapping {
     my $image = $F->load_image(
         {
             'x'      => 0,
-            'y'      => 0,
-            'width'  => $XX * .75,
-            'height' => ($YY - $F->{'Y_CLIP'}) * .75,
+            'y'      => $F->{'Y_CLIP'},
+            'width'  => $XX,
+            'height' => $YY - $F->{'Y_CLIP'},
             'center' => CENTER_XY,
             'file'   => "$images_path/Test_Pattern.jpg"
         }
@@ -1692,7 +1692,7 @@ sub print_it {
         my $b = $fb->ttf_print(
             {
                 'x'            => 5 * $xm,
-                'y'            => max(9, 15 * $ym),
+                'y'            => max(18, 25 * $ym),
                 'height'       => max(9, 20 * $ym),
                 'color'        => $color,
                 'text'         => $message,
