@@ -11,6 +11,10 @@ use constant {
     TRUE        => 1,
     FALSE       => 0,
 
+    HARDWARE    => 2,
+    SOFTWARE    => 1,
+    PERL        => 0,
+
     NORMAL_MODE => 0,    #   Constants for DRAW_MODE
     XOR_MODE    => 1,    #   Constants for DRAW_MODE
     OR_MODE     => 2,    #   Constants for DRAW_MODE
@@ -66,6 +70,7 @@ use constant {
     FBINFO_HWACCEL_YWRAP     => 0x4000,
 };
 
+use POSIX ();
 use List::Util qw(min max);
 
 # use Inline (Config => directory => './.Graphics-Framebuffer-Inline');
@@ -100,12 +105,12 @@ eval {
 
 unsigned int c_get_screen_info(char *fb_file) {
     int fbfd = 0;
-/*    struct fb_var_screeninfo vinfo;*/
+/*  struct fb_var_screeninfo vinfo; */
     struct fb_fix_screeninfo finfo;
     
     fbfd = open(fb_file,O_RDWR);
     ioctl(fbfd, FBIOGET_FSCREENINFO, &finfo);
-/*    ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo);*/
+/*  ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo); */
     close(fbfd);
     return(finfo.line_length);
 }
@@ -115,6 +120,7 @@ void c_plot(
     short x, short y,
     unsigned char draw_mode,
     unsigned int color,
+    unsigned int bcolor,
     unsigned char bytes_per_pixel,
     unsigned int bytes_per_line,
     unsigned short x_clip, unsigned short y_clip, unsigned short xx_clip, unsigned short yy_clip,
@@ -130,8 +136,8 @@ void c_plot(
               if (bytes_per_pixel == 4) {
                  *((unsigned int*)(framebuffer + index)) = color;
               } else if (bytes_per_pixel == 3) {
-                  *(framebuffer + index)     = color & 255;
-                  *(framebuffer + index + 1) = (color >> 8) & 255;
+                  *(framebuffer + index)     = color         & 255;
+                  *(framebuffer + index + 1) = (color >> 8)  & 255;
                   *(framebuffer + index + 2) = (color >> 16) & 255;
               } else {
                   *((unsigned short*)(framebuffer + index)) = (short) color;
@@ -141,8 +147,8 @@ void c_plot(
               if (bytes_per_pixel == 4) {
                   *((unsigned int*)(framebuffer + index)) ^= color;
               } else if (bytes_per_pixel == 3) {
-                  *(framebuffer + index)     ^= color & 255;
-                  *(framebuffer + index + 1) ^= (color >> 8) & 255;
+                  *(framebuffer + index)     ^= color         & 255;
+                  *(framebuffer + index + 1) ^= (color >> 8)  & 255;
                   *(framebuffer + index + 2) ^= (color >> 16) & 255;
               } else {
                   *((unsigned short*)(framebuffer + index)) ^= (short) color;
@@ -152,8 +158,8 @@ void c_plot(
               if (bytes_per_pixel == 4) {
                   *((unsigned int*)(framebuffer + index)) |= color;
               } else if (bytes_per_pixel == 3) {
-                  *(framebuffer + index)     |= color & 255;
-                  *(framebuffer + index + 1) |= (color >> 8) & 255;
+                  *(framebuffer + index)     |= color         & 255;
+                  *(framebuffer + index + 1) |= (color >> 8)  & 255;
                   *(framebuffer + index + 2) |= (color >> 16) & 255;
               } else {
                   *((unsigned short*)(framebuffer + index)) |= (short) color;
@@ -163,26 +169,62 @@ void c_plot(
               if (bytes_per_pixel == 4) {
                   *((unsigned int*)(framebuffer + index)) &= color;
               } else if (bytes_per_pixel == 3) {
-                  *(framebuffer + index)     &= color & 255;
-                  *(framebuffer + index + 1) &= (color >> 8) & 255;
+                  *(framebuffer + index)     &= color         & 255;
+                  *(framebuffer + index + 1) &= (color >> 8)  & 255;
                   *(framebuffer + index + 2) &= (color >> 16) & 255;
               } else {
                   *((unsigned short*)(framebuffer + index)) &= (short) color;
               }
               break;
             case MASK_MODE :
+              if (bytes_per_pixel == 4) {
+                  unsigned int rgb = *((unsigned int*)(framebuffer + index ));
+                  if ((rgb & 0xFFFFFF00) != (bcolor & 0xFFFFFF00)) { // Ignore alpha channel
+                      *((unsigned int*)(framebuffer + index )) = color;
+                  }
+              } else if (bytes_per_pixel == 3) {
+                  unsigned int rgb = *((unsigned int*)(framebuffer + index )) & 0xFFFFFF00;
+                  if (rgb != (bcolor & 0xFFFFFF00)) { // Ignore alpha channel
+                      *(framebuffer + index )     = color         & 255;
+                      *(framebuffer + index  + 1) = (color >> 8)  & 255;
+                      *(framebuffer + index  + 2) = (color >> 16) & 255;
+                  }
+              } else {
+                  unsigned short rgb = *((unsigned short*)(framebuffer + index));
+                  if (rgb != (bcolor & 0xFFFF)) {
+                      *((unsigned short*)(framebuffer + index )) = color;
+                  }
+              }
               break;
             case UNMASK_MODE :
+              if (bytes_per_pixel == 4) {
+                  unsigned int rgb = *((unsigned int*)(framebuffer + index ));
+                  if ((rgb & 0xFFFFFF00) == (bcolor & 0xFFFFFF00)) { // Ignore alpha channel
+                      *((unsigned int*)(framebuffer + index )) = color;
+                  }
+              } else if (bytes_per_pixel == 3) {
+                  unsigned int rgb = *((unsigned int*)(framebuffer + index )) & 0xFFFFFF00;
+                  if (rgb == (bcolor & 0xFFFFFF00)) { // Ignore alpha channel
+                      *(framebuffer + index )     = color         & 255;
+                      *(framebuffer + index  + 1) = (color >> 8)  & 255;
+                      *(framebuffer + index  + 2) = (color >> 16) & 255;
+                  }
+              } else {
+                  unsigned short rgb = *((unsigned short*)(framebuffer + index));
+                  if (rgb == (bcolor & 0xFFFF)) {
+                      *((unsigned short*)(framebuffer + index )) = color;
+                  }
+              }
               break;
             case ALPHA_MODE :
               if (bytes_per_pixel >= 3) {
                   unsigned short fb_r = *(framebuffer + index);
                   unsigned short fb_g = *(framebuffer + index + 1);
                   unsigned short fb_b = *(framebuffer + index + 2);
-                  unsigned char invA = (255 - alpha);
-                  unsigned char R = color & 255;
-                  unsigned char G = (color >> 8) & 255;
-                  unsigned char B = (color >> 16) & 255;
+                  unsigned char invA  = (255 - alpha);
+                  unsigned char R     = color         & 255;
+                  unsigned char G     = (color >> 8)  & 255;
+                  unsigned char B     = (color >> 16) & 255;
 
                   fb_r = ((R * alpha) + (fb_r * invA)) >> 8;
                   fb_g = ((G * alpha) + (fb_g * invA)) >> 8;
@@ -193,22 +235,22 @@ void c_plot(
                   *(framebuffer + index + 2) = fb_b;
               } else {
                   unsigned short rgb565 = *((unsigned short*)(framebuffer + index));
-                  unsigned short fb_r = rgb565;
-                  unsigned short fb_g = rgb565;
-                  unsigned short fb_b = rgb565;
+                  unsigned short fb_r   = rgb565;
+                  unsigned short fb_g   = rgb565;
+                  unsigned short fb_b   = rgb565;
                   fb_b >>= 11;
                   fb_g >>= 5;
-                  fb_r &= 31;
-                  fb_g &= 63;
-                  fb_b &= 31;
+                  fb_r  &= 31;
+                  fb_g  &= 63;
+                  fb_b  &= 31;
                   unsigned short R = color;
                   unsigned short G = color;
                   unsigned short B = color;
                   B >>= 11;
                   G >>= 5;
-                  R &= 31;
-                  G &= 63;
-                  B &= 31;
+                  R  &= 31;
+                  G  &= 63;
+                  B  &= 31;
                   unsigned char invA = (255 - alpha);
                   fb_r = ((R * alpha) + (fb_r * invA)) >> 8;
                   fb_g = ((G * alpha) + (fb_g * invA)) >> 8;
@@ -226,127 +268,61 @@ void c_line(
     short x1, short y1, short x2, short y2,
     unsigned char draw_mode,
     unsigned int color,
+    unsigned int bcolor,
     unsigned char bytes_per_pixel,
     unsigned int bytes_per_line,
     unsigned short x_clip, unsigned short y_clip, unsigned short xx_clip, unsigned short yy_clip,
     unsigned short xoffset, unsigned short yoffset,
     unsigned char alpha
 ) {
-/*    printf("X=%d, Y=%d, XX=%d, YY=%d, DM=%d, C=%d, BPP=%d, BPL=%d, XC=%d, YC=%d, XXC=%d, YYC=%d, XO=%d, YO=%d, A=%d\\n", x1,y1,x2,y2,draw_mode,color,bytes_per_pixel,bytes_per_line,x_clip,y_clip,xx_clip,yy_clip,xoffset,yoffset,alpha);
+        int shortLen = y2 - y1;
+        int longLen  = x2 - x1;
+        int yLonger  = false;
 
-    int cx, cy,
-      ix, iy,
-      dx, dy, 
-      ddx = x2 - x1, ddy = y2 - y1;
-    
-    if (!ddx) { //vertical line special case
-          if (ddy > 0) {
-              cy = y1;  
-              do
-                c_plot(framebuffer, x1, cy++, draw_mode, color, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
-              while (cy <= y2);
-              return;
-          } else {
-              cy = y2;
-              do
-                c_plot(framebuffer, x1, cy++, draw_mode, color, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
-              while (cy <= y1);
-              return;
-          }
-    }
-    if (!ddy) { //horizontal line special case
-          if (ddx > 0) {
-              cx = x1;
-              do
-                c_plot(framebuffer, cx, y1, draw_mode, color, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
-              while (++cx <= x2);
-              return;
-          } else {
-              cx = x2; 
-              do
-                c_plot(framebuffer, cx, y1, draw_mode, color, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
-              while (++cx <= x1);
-              return;
-          }
-    }
-    if (ddy < 0) { iy= -1; ddy= -ddy; }//pointing up
-      else iy= 1;
-    if (ddx < 0) { ix= -1; ddx= -ddx; }//pointing left
-      else ix= 1;
-    dx = dy = ddx * ddy;
-    cy = y1, cx = x1; 
-    if (ddx < ddy) { // < 45 degrees, a tall line    
-          do {
-              dx -= ddy;
-              do {
-                c_plot(framebuffer, cx, cy, draw_mode, color, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
-                  cy += iy, dy -= ddx;
-              } while (dy >=dx);
-              cx += ix;
-          } while (dx > 0);
-    } else { // >= 45 degrees, a wide line
-          do {
-              dy -= ddx;
-              do {
-                c_plot(framebuffer, cx, cy, draw_mode, color, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
-                  cx += ix, dx -= ddy;
-              } while (dx >=dy);
-              cy += iy;
-          } while (dy > 0);
-    }
-*/    
-    
-    
-    int shortLen = y2 - y1;
-    int longLen  = x2 - x1;
-    int yLonger  = false;
-
-    if (abs(shortLen) > abs(longLen)) {
-        int swap = shortLen;
-        shortLen   = longLen;
-        longLen    = swap;                
-        yLonger    = true;
-    }
-    int decInc;
-    if (longLen == 0) {
-        decInc = 0;
-    } else {
-        decInc = (shortLen << 16) / longLen;
-    }
-    int count;
-    if (yLonger) {
-        if (longLen > 0) {
+        if (abs(shortLen) > abs(longLen)) {
+            int swap = shortLen;
+            shortLen   = longLen;
+            longLen    = swap;                
+            yLonger    = true;
+        }
+        int decInc;
+        if (longLen == 0) {
+            decInc = 0;
+        } else {
+            decInc = (shortLen << 16) / longLen;
+        }
+        int count;
+        if (yLonger) {
+            if (longLen > 0) {
+                longLen += y1;
+                for (count = 0x8000 + (x1 << 16); y1 <= longLen; ++y1) {
+                    c_plot(framebuffer, count >> 16, y1, draw_mode, color, bcolor, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
+                    count += decInc;
+                }
+                return;
+            }
             longLen += y1;
-            for (count = 0x8000 + (x1 << 16); y1 <= longLen; ++y1) {
-                c_plot(framebuffer, count >> 16, y1, draw_mode, color, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
+            for (count = 0x8000 + (x1 << 16); y1 >= longLen; --y1) {
+                c_plot(framebuffer, count >> 16, y1, draw_mode, color, bcolor, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
+                count -= decInc;
+            }
+            return;    
+        }
+ 
+        if (longLen > 0) {
+            longLen += x1;
+            for (count = 0x8000 + (y1 << 16); x1 <= longLen; ++x1) {
+                c_plot(framebuffer, x1, count >> 16, draw_mode, color, bcolor, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
                 count += decInc;
             }
             return;
         }
-        longLen += y1;
-        for (count = 0x8000 + (x1 << 16); y1 >= longLen; --y1) {
-            c_plot(framebuffer, count >> 16, y1, draw_mode, color, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
+        longLen += x1;
+        for (count = 0x8000 + (y1 << 16); x1 >= longLen; --x1) {
+            c_plot(framebuffer, x1, count >> 16, draw_mode, color, bcolor, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
             count -= decInc;
         }
-        return;    
-    }
-    
-    if (longLen > 0) {
-        longLen += x1;
-        for (count = 0x8000 + (y1 << 16); x1 <= longLen; ++x1) {
-            c_plot(framebuffer, x1, count >> 16, draw_mode, color, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
-            count += decInc;
-        }
-        return;
-    }
-    longLen += x1;
-    for (count = 0x8000 + (y1 << 16); x1 >= longLen; --x1) {
-        c_plot(framebuffer, x1, count >> 16, draw_mode, color, bytes_per_pixel, bytes_per_line, x_clip, y_clip, xx_clip, yy_clip, xoffset, yoffset, alpha);
-        count -= decInc;
-    }
-
 }
-
 void c_blit_write(char *framebuffer, unsigned short screen_width, unsigned short screen_height, unsigned int bytes_per_line, unsigned short xoffset, unsigned short yoffset, char *blit_data, short x, short y, unsigned short w, unsigned short h, unsigned char bytes_per_pixel, unsigned char draw_mode, unsigned char alpha, unsigned int bcolor, unsigned short x_clip, unsigned short y_clip, unsigned short xx_clip, unsigned short yy_clip) {
     short fb_x = xoffset + x;
     short fb_y = yoffset + y;
@@ -366,6 +342,7 @@ void c_blit_write(char *framebuffer, unsigned short screen_width, unsigned short
         }
     } else {
         for (vertical = 0; vertical < h; vertical++) {
+            unsigned int vbl  = vertical * bline;
             unsigned short yv = fb_y + vertical;
             unsigned int yvbl = yv * bytes_per_line;
             if (yv >= (yoffset + y_clip) && yv <= (yoffset + yy_clip)) {
@@ -373,102 +350,105 @@ void c_blit_write(char *framebuffer, unsigned short screen_width, unsigned short
                     unsigned short xh = fb_x + horizontal;
                     unsigned int xhbp = xh * bytes_per_pixel;
                     if (xh >= (xoffset + x_clip) && xh <= (xoffset + xx_clip)) {
-                        unsigned int hzpixel = horizontal * bytes_per_pixel;
+                        unsigned int hzpixel   = horizontal * bytes_per_pixel;
+                        unsigned int vhz       = vbl + hzpixel;
+                        unsigned int yvhz      = yvbl + hzpixel;
+                        unsigned int xhbp_yvbl = xhbp + yvbl;
                         switch(draw_mode) {
                             case NORMAL_MODE :
                               if (bytes_per_pixel == 4) {
-                                  *((unsigned int*)(framebuffer + xhbp + yvbl)) = *((unsigned int*)(blit_data + (vertical * bline) + hzpixel));
+                                  *((unsigned int*)(framebuffer + xhbp_yvbl)) = *((unsigned int*)(blit_data + vhz));
                               } else if (bytes_per_pixel == 3) {
-                                  *(framebuffer + xhbp + yvbl )     = *(blit_data + (vertical * bline) + hzpixel );
-                                  *(framebuffer + xhbp + yvbl  + 1) = *(blit_data + (vertical * bline) + hzpixel  + 1);
-                                  *(framebuffer + xhbp + yvbl  + 2) = *(blit_data + (vertical * bline) + hzpixel  + 2);
+                                  *(framebuffer + xhbp_yvbl )     = *(blit_data + vhz );
+                                  *(framebuffer + xhbp_yvbl  + 1) = *(blit_data + vhz  + 1);
+                                  *(framebuffer + xhbp_yvbl  + 2) = *(blit_data + vhz  + 2);
                               } else {
-                                  *((unsigned short*)(framebuffer + xhbp + yvbl )) = *((unsigned short*)(blit_data + (vertical * bline) + hzpixel ));
+                                  *((unsigned short*)(framebuffer + xhbp_yvbl )) = *((unsigned short*)(blit_data + vhz ));
                               }
                             break;
                             case XOR_MODE :
                               if (bytes_per_pixel == 4) {
-                                  *((unsigned int*)(framebuffer + xhbp + yvbl )) ^= *((unsigned int*)(blit_data + (vertical * bline) + hzpixel ));
+                                  *((unsigned int*)(framebuffer + xhbp_yvbl )) ^= *((unsigned int*)(blit_data + vhz ));
                               } else if (bytes_per_pixel == 3) {
-                                  *(framebuffer + xhbp + yvbl )     ^= *(blit_data + (vertical * bline) + hzpixel );
-                                  *(framebuffer + xhbp + yvbl  + 1) ^= *(blit_data + (vertical * bline) + hzpixel  + 1);
-                                  *(framebuffer + xhbp + yvbl  + 2) ^= *(blit_data + (vertical * bline) + hzpixel  + 2);
+                                  *(framebuffer + xhbp_yvbl )     ^= *(blit_data + vhz );
+                                  *(framebuffer + xhbp_yvbl  + 1) ^= *(blit_data + vhz  + 1);
+                                  *(framebuffer + xhbp_yvbl  + 2) ^= *(blit_data + vhz  + 2);
                               } else {
-                                  *((unsigned short*)(framebuffer + xhbp + yvbl )) ^= *((unsigned short*)(blit_data + (vertical * bline) + hzpixel ));
+                                  *((unsigned short*)(framebuffer + xhbp_yvbl )) ^= *((unsigned short*)(blit_data + vhz ));
                               }
                             break;
                             case OR_MODE :
                               if (bytes_per_pixel == 4) {
-                                  *((unsigned int*)(framebuffer + xhbp + yvbl )) |= *((unsigned int*)(blit_data + (vertical * bline) + hzpixel ));
+                                  *((unsigned int*)(framebuffer + xhbp_yvbl )) |= *((unsigned int*)(blit_data + vhz ));
                               } else if (bytes_per_pixel == 3) {
-                                  *(framebuffer + xhbp + yvbl )     |= *(blit_data + (vertical * bline) + hzpixel );
-                                  *(framebuffer + xhbp + yvbl  + 1) |= *(blit_data + (vertical * bline) + hzpixel  + 1);
-                                  *(framebuffer + xhbp + yvbl  + 2) |= *(blit_data + (vertical * bline) + hzpixel  + 2);
+                                  *(framebuffer + xhbp_yvbl )     |= *(blit_data + vhz );
+                                  *(framebuffer + xhbp_yvbl  + 1) |= *(blit_data + vhz  + 1);
+                                  *(framebuffer + xhbp_yvbl  + 2) |= *(blit_data + vhz  + 2);
                               } else {
-                                  *((unsigned short*)(framebuffer + xhbp + yvbl )) |= *((unsigned short*)(blit_data + (vertical * bline) + hzpixel ));
+                                  *((unsigned short*)(framebuffer + xhbp_yvbl )) |= *((unsigned short*)(blit_data + vhz ));
                               }
                             break;
                             case AND_MODE :
                               if (bytes_per_pixel == 4) {
-                                  *((unsigned int*)(framebuffer + xhbp + yvbl )) &= *((unsigned int*)(blit_data + (vertical * bline) + hzpixel ));
+                                  *((unsigned int*)(framebuffer + xhbp_yvbl )) &= *((unsigned int*)(blit_data + vhz ));
                               } else if (bytes_per_pixel == 3) {
-                                  *(framebuffer + xhbp + yvbl )     &= *(blit_data + (vertical * bline) + hzpixel );
-                                  *(framebuffer + xhbp + yvbl  + 1) &= *(blit_data + (vertical * bline) + hzpixel  + 1);
-                                  *(framebuffer + xhbp + yvbl  + 2) &= *(blit_data + (vertical * bline) + hzpixel  + 2);
+                                  *(framebuffer + xhbp_yvbl )     &= *(blit_data + vhz );
+                                  *(framebuffer + xhbp_yvbl  + 1) &= *(blit_data + vhz  + 1);
+                                  *(framebuffer + xhbp_yvbl  + 2) &= *(blit_data + vhz  + 2);
                               } else {
-                                  *((unsigned short*)(framebuffer + xhbp + yvbl )) &= *((unsigned short*)(blit_data + (vertical * bline) + hzpixel ));
+                                  *((unsigned short*)(framebuffer + xhbp_yvbl )) &= *((unsigned short*)(blit_data + vhz ));
                               }
                             break;
                             case MASK_MODE :
                               if (bytes_per_pixel == 4) {
-                                  unsigned int rgb = *((unsigned int*)(blit_data + (vertical * bline) + hzpixel ));
+                                  unsigned int rgb = *((unsigned int*)(blit_data + vhz ));
                                   if ((rgb & 0xFFFFFF00) != (bcolor & 0xFFFFFF00)) { // Ignore alpha channel
-                                        *((unsigned int*)(framebuffer + xhbp + yvbl )) = rgb;
+                                      *((unsigned int*)(framebuffer + xhbp_yvbl )) = rgb;
                                   }
                               } else if (bytes_per_pixel == 3) {
-                                  unsigned int rgb = *((unsigned int*)(blit_data + (vertical * bline) + hzpixel )) & 0xFFFFFF00;
+                                  unsigned int rgb = *((unsigned int*)(blit_data + vhz )) & 0xFFFFFF00;
                                   if (rgb != (bcolor & 0xFFFFFF00)) { // Ignore alpha channel
-                                        *(framebuffer + xhbp + yvbl )     = *(blit_data + (vertical * bline) + hzpixel );
-                                      *(framebuffer + xhbp + yvbl  + 1) = *(blit_data + (vertical * bline) + hzpixel  + 1);
-                                      *(framebuffer + xhbp + yvbl  + 2) = *(blit_data + (vertical * bline) + hzpixel  + 2);
+                                      *(framebuffer + xhbp_yvbl )     = *(blit_data + vhz );
+                                      *(framebuffer + xhbp_yvbl  + 1) = *(blit_data + vhz  + 1);
+                                      *(framebuffer + xhbp_yvbl  + 2) = *(blit_data + vhz  + 2);
                                   }
                               } else {
-                                  unsigned short rgb = *((unsigned short*)(blit_data + (vertical * bline) + hzpixel ));
+                                  unsigned short rgb = *((unsigned short*)(blit_data + vhz ));
                                   if (rgb != (bcolor & 0xFFFF)) {
-                                      *((unsigned short*)(framebuffer + xhbp + yvbl )) = rgb;
+                                      *((unsigned short*)(framebuffer + xhbp_yvbl )) = rgb;
                                   }
                               }
                             break;
                             case UNMASK_MODE :
                               if (bytes_per_pixel == 4) {
-                                  unsigned int rgb = *((unsigned int*)(framebuffer + xhbp + yvbl ));
+                                  unsigned int rgb = *((unsigned int*)(framebuffer + xhbp_yvbl ));
                                   if ((rgb & 0xFFFFFF00) == (bcolor & 0xFFFFFF00)) { // Ignore alpha channel
-                                        *((unsigned int*)(framebuffer + xhbp + yvbl )) = *((unsigned int*)(blit_data + (vertical * bline) + hzpixel ));
+                                      *((unsigned int*)(framebuffer + xhbp_yvbl )) = *((unsigned int*)(blit_data + vhz ));
                                   }
                               } else if (bytes_per_pixel == 3) {
-                                  unsigned int rgb = *((unsigned int*)(framebuffer + xhbp + yvbl + hzpixel ));
+                                  unsigned int rgb = *((unsigned int*)(framebuffer + xhbp + yvhz ));
                                   if (rgb == (bcolor & 0xFFFFFF00)) { // Ignore alpha channel
-                                        *(framebuffer + xhbp + yvbl )     = *(blit_data + (vertical * bline) + hzpixel );
-                                      *(framebuffer + xhbp + yvbl  + 1) = *(blit_data + (vertical * bline) + hzpixel  + 1);
-                                      *(framebuffer + xhbp + yvbl  + 2) = *(blit_data + (vertical * bline) + hzpixel  + 2);
+                                      *(framebuffer + xhbp_yvbl )     = *(blit_data + vhz );
+                                      *(framebuffer + xhbp_yvbl  + 1) = *(blit_data + vhz  + 1);
+                                      *(framebuffer + xhbp_yvbl  + 2) = *(blit_data + vhz  + 2);
                                   }
                               } else {
-                                  unsigned short rgb = *((unsigned short*)(framebuffer + xhbp + yvbl + hzpixel ));
+                                  unsigned short rgb = *((unsigned short*)(framebuffer + xhbp + yvhz ));
                                   if (rgb == (bcolor & 0xFFFF)) {
-                                      *((unsigned short*)(framebuffer + xhbp + yvbl )) = *((unsigned short*)(blit_data + (vertical * bline) + hzpixel ));
+                                      *((unsigned short*)(framebuffer + xhbp_yvbl )) = *((unsigned short*)(blit_data + vhz ));
                                   }
                               }
                             break;
                             case ALPHA_MODE :
                               if (bytes_per_pixel == 4) {
-                                  unsigned char fb_r = *(framebuffer + xhbp + yvbl );
-                                  unsigned char fb_g = *(framebuffer + xhbp + yvbl + 1);
-                                  unsigned char fb_b = *(framebuffer + xhbp + yvbl + 2);
-                                  unsigned char fb_a = *(framebuffer + xhbp + yvbl + 3);
-                                  unsigned char R    = *(blit_data + (vertical * bline) + hzpixel );
-                                  unsigned char G    = *(blit_data + (vertical * bline) + hzpixel  + 1);
-                                  unsigned char B    = *(blit_data + (vertical * bline) + hzpixel  + 2);
-                                  unsigned char A    = *(blit_data + (vertical * bline) + hzpixel  + 3);
+                                  unsigned char fb_r = *(framebuffer + xhbp_yvbl );
+                                  unsigned char fb_g = *(framebuffer + xhbp_yvbl + 1);
+                                  unsigned char fb_b = *(framebuffer + xhbp_yvbl + 2);
+                                  unsigned char fb_a = *(framebuffer + xhbp_yvbl + 3);
+                                  unsigned char R    = *(blit_data + vhz );
+                                  unsigned char G    = *(blit_data + vhz  + 1);
+                                  unsigned char B    = *(blit_data + vhz  + 2);
+                                  unsigned char A    = *(blit_data + vhz  + 3);
                                   unsigned char invA = (255 - A);
 
                                   fb_r = ((R * A) + (fb_r * invA)) >> 8;
@@ -476,28 +456,28 @@ void c_blit_write(char *framebuffer, unsigned short screen_width, unsigned short
                                   fb_b = ((B * A) + (fb_b * invA)) >> 8;
                                   fb_a = (fb_a + A) & 255;
 
-                                  *(framebuffer + xhbp + yvbl )     = fb_r;
-                                  *(framebuffer + xhbp + yvbl  + 1) = fb_g;
-                                  *(framebuffer + xhbp + yvbl  + 2) = fb_b;
-                                  *(framebuffer + xhbp + yvbl  + 3) = fb_a;
+                                  *(framebuffer + xhbp_yvbl )     = fb_r;
+                                  *(framebuffer + xhbp_yvbl  + 1) = fb_g;
+                                  *(framebuffer + xhbp_yvbl  + 2) = fb_b;
+                                  *(framebuffer + xhbp_yvbl  + 3) = fb_a;
                               } else if (bytes_per_pixel == 3) {
-                                  unsigned char fb_r = *(framebuffer + xhbp + yvbl );
-                                  unsigned char fb_g = *(framebuffer + xhbp + yvbl  + 1);
-                                  unsigned char fb_b = *(framebuffer + xhbp + yvbl  + 2);
-                                  unsigned char R    = *(blit_data + (vertical * bline) + hzpixel );
-                                  unsigned char G    = *(blit_data + (vertical * bline) + hzpixel + 1);
-                                  unsigned char B    = *(blit_data + (vertical * bline) + hzpixel + 2);
+                                  unsigned char fb_r = *(framebuffer + xhbp_yvbl );
+                                  unsigned char fb_g = *(framebuffer + xhbp_yvbl  + 1);
+                                  unsigned char fb_b = *(framebuffer + xhbp_yvbl  + 2);
+                                  unsigned char R    = *(blit_data + vhz );
+                                  unsigned char G    = *(blit_data + vhz + 1);
+                                  unsigned char B    = *(blit_data + vhz + 2);
                                   unsigned char invA = (255 - alpha);
 
                                   fb_r = ((R * alpha) + (fb_r * invA)) >> 8;
                                   fb_g = ((G * alpha) + (fb_g * invA)) >> 8;
                                   fb_b = ((B * alpha) + (fb_b * invA)) >> 8;
 
-                                  *(framebuffer + xhbp + yvbl )     = fb_r;
-                                  *(framebuffer + xhbp + yvbl  + 1) = fb_g;
-                                  *(framebuffer + xhbp + yvbl  + 2) = fb_b;
+                                  *(framebuffer + xhbp_yvbl )     = fb_r;
+                                  *(framebuffer + xhbp_yvbl  + 1) = fb_g;
+                                  *(framebuffer + xhbp_yvbl  + 2) = fb_b;
                               } else {
-                                  unsigned short rgb565 = *((unsigned short*)(framebuffer + xhbp + yvbl ));
+                                  unsigned short rgb565 = *((unsigned short*)(framebuffer + xhbp_yvbl ));
 
                                   unsigned short fb_r = rgb565;
                                   unsigned short fb_g = rgb565;
@@ -507,7 +487,7 @@ void c_blit_write(char *framebuffer, unsigned short screen_width, unsigned short
                                   fb_r &= 31;
                                   fb_g &= 63;
                                   fb_b &= 31;
-                                  rgb565 = *((unsigned short*)(blit_data + (vertical * bline) + hzpixel ));
+                                  rgb565 = *((unsigned short*)(blit_data + vhz ));
                                   unsigned short R = rgb565;
                                   unsigned short G = rgb565;
                                   unsigned short B = rgb565;
@@ -523,7 +503,7 @@ void c_blit_write(char *framebuffer, unsigned short screen_width, unsigned short
                                   rgb565 = 0;
                                   rgb565 = (fb_b << 11) | (fb_g << 5) | fb_r;
 
-                                  *((unsigned short*)(framebuffer + xhbp + yvbl )) = rgb565;
+                                  *((unsigned short*)(framebuffer + xhbp_yvbl )) = rgb565;
                               }
                             break;
                         }
@@ -726,7 +706,7 @@ END_C
 
 BEGIN {
     require Exporter;
-    our $VERSION = 1.01;
+    our $VERSION = 1.02;
     our @EXPORT_OK = qw(c_monochrome c_convert_16_24 c_convert_24_16 c_get_screen_info c_plot c_line c_blit_write c_flip_vertical c_flip_horizontal c_rotate );
     our @EXPORT    = qw( $INLINE );
 }
@@ -1048,16 +1028,16 @@ sub blit_write {
         return;
     }
 
-    my $max = $self->{'fscreeninfo'}->{'smem_len'} - $bytes;
-    my $scan           = $w * $bytes;
-    my $yend           = $y + $h;
+    my $max  = $self->{'fscreeninfo'}->{'smem_len'} - $bytes;
+    my $scan = $w * $bytes;
+    my $yend = $y + $h;
 
     #    my $WW = $scan * $h;
     my $WW = int((length($scrn) / $h));
     my $X_X = ($x + $self->{'XOFFSET'}) * $bytes;
     my ($index, $data, $px, $line, $idx, $px4, $buf);
 
-    $idx = 0;
+    $idx   = 0;
     $y    += $self->{'YOFFSET'};
     $yend += $self->{'YOFFSET'};
 
@@ -1496,7 +1476,7 @@ sub blit_transform {
             }
         }
         return ($params->{'blit_data'}) if (abs($degrees) == 360 || $degrees == 0);    # 0 and 360 are not a rotation
-        unless ($params->{'rotate'}->{'quality'} eq 'high' || ! $self->{'ACCELERATED'}) {
+        unless ($params->{'rotate'}->{'quality'} eq 'high' || $self->{'ACCELERATED'} == PERL) {
              if (abs($degrees) == 180) {
                 my $new = "$image";
                 c_flip_both($new,$width,$height,$bytes);
@@ -1686,13 +1666,36 @@ sub blit_copy {
     my $xx = int($params->{'x_dest'});
     my $yy = int($params->{'y_dest'});
 
-    # If acceleration isn't working, then just set 'ACCELERATED' to zero.
-#    if ($self->{'ACCELERATED'} && $self->{'DRAW_MODE'} < 1) {
+    if ($self->{'ACCELERATED'} == HARDWARE && $self->{'DRAW_MODE'} < 1) {
+        # accelerated_blit_copy($self->{'FB'}, $x, $y, $w, $h, $xx, $yy);
+    } else {
+        $self->blit_write({ %{ $self->blit_read({ 'x' => $x, 'y' => $y, 'width' => $w, 'height' => $h }) }, 'x' => $xx, 'y' => $yy });
+    }
+} ## end sub blit_copy
 
-    # accelerated_blit_copy($self->{'FB'}, $x, $y, $w, $h, $xx, $yy);
-#    } else {
-    $self->blit_write({ %{ $self->blit_read({ 'x' => $x, 'y' => $y, 'width' => $w, 'height' => $h }) }, 'x' => $xx, 'y' => $yy });
-#    }
+sub blit_move {
+    my $self   = shift;
+    my $params = shift;
+
+    my $x  = int($params->{'x'});
+    my $y  = int($params->{'y'});
+    my $w  = int($params->{'width'});
+    my $h  = int($params->{'height'});
+    my $xx = int($params->{'x_dest'});
+    my $yy = int($params->{'y_dest'});
+
+    if ($self->{'ACCELERATED'} == HARDWARE && $self->{'DRAW_MODE'} == NORMAL_MODE) {
+        # accelerated_blit_move($self->{'FB'}, $x, $y, $w, $h, $xx, $yy);
+    } else {
+        my $old_mode = $self->{'DRAW_MODE'};
+        my $image    = $self->blit_read({ 'x' => $x, 'y' => $y, 'width' => $w, 'height' => $h });
+        $self->xor_mode();
+        $self->blit_write($image);
+        $self->{'DRAW_MODE'} = $old_mode;
+        $image->{'x'}        = $xx;
+        $image->{'y'}        = $yy;
+        $self->blit_write($image);
+    }
 } ## end sub blit_copy
 
 sub plot {
@@ -1719,6 +1722,7 @@ sub plot {
                 $x, $y,
                 $self->{'DRAW_MODE'},
                 $self->{'INT_COLOR'},
+                $self->{'INT_B_COLOR'},
                 $self->{'BYTES'},
                 $self->{'BYTES_PER_LINE'},
                 $self->{'X_CLIP'},
@@ -1837,17 +1841,18 @@ sub drawto {
     my $XX          = $x_end;
     my $YY          = $y_end;
 
-    if ($self->{'ACCELERATED'} && ! $self->{'FILE_MODE'} && ! $antialiased) {
+    if ($self->{'ACCELERATED'} && $size == 1 && ! $self->{'FILE_MODE'} && ! $antialiased) {
         c_line(
             $self->{'SCREEN'},
             $start_x, $start_y, $x_end, $y_end,
             $self->{'DRAW_MODE'},
             $self->{'INT_COLOR'},
+            $self->{'INT_B_COLOR'},
             $self->{'BYTES'},
             $self->{'BYTES_PER_LINE'},
             $self->{'X_CLIP'}, $self->{'Y_CLIP'}, $self->{'XX_CLIP'}, $self->{'YY_CLIP'},
             $self->{'XOFFSET'}, $self->{'YOFFSET'},
-            $self->{'COLOR_ALPHA'}
+            $self->{'COLOR_ALPHA'},
         );
     } else {
     
@@ -1988,13 +1993,23 @@ sub drawto {
 } ## end sub drawto
 
 sub _adj_plot {
-    my ($self, $x, $y, $c, $s) = @_;
+    my $self = shift;
+    my $x    = shift;
+    my $y    = shift;
+    my $c    = shift;
+    my $s    = shift;
+#    my ($self, $x, $y, $c, $s) = @_;
     $self->set_color({ 'red' => $s->{'red'} * $c, 'green' => $s->{'green'} * $c, 'blue' => $s->{'blue'} * $c });
     $self->plot({ 'x' => $x, 'y' => $y });
 }
 
 sub _draw_line_antialiased {
-    my ($self, $x0, $y0, $x1, $y1) = @_;
+    my $self = shift;
+    my $x0   = shift;
+    my $y0   = shift;
+    my $x1   = shift;
+    my $y1   = shift;
+#    my ($self, $x0, $y0, $x1, $y1) = @_;
 
     my $saved = { %{ $self->{'SET_COLOR'} } };
 
@@ -2017,9 +2032,9 @@ sub _draw_line_antialiased {
     my $intery;
 
     # handle the endpoints
-    for my $xy ([$x0, $y0], [$x1, $y1]) {
+    foreach my $xy ([$x0, $y0], [$x1, $y1]) {
         my ($x, $y) = @{$xy};
-        my $xend    = _round($x);
+        my $xend    = POSIX::lround($x);
         my $yend    = $y + $gradient * ($xend - $x);
         my $xgap    = _rfpart($x + 0.5);
 
@@ -2037,20 +2052,16 @@ sub _draw_line_antialiased {
 
     # main loop
 
-    for my $x ($xends[0] + 1 .. $xends[1] - 1) {
+    foreach my $x ($xends[0] + 1 .. $xends[1] - 1) {
         $plot->($self, $x, int($intery),     _rfpart($intery), $saved);
         $plot->($self, $x, int($intery) + 1, _fpart($intery),  $saved);
         $intery += $gradient;
     }
     $self->set_color($saved);
 } ## end sub _draw_line_antialiased
-sub _round {
-    return (int(0.5 + shift));
-}
 
 sub _fpart {
-    my $x = shift;
-    return ($x - int $x);
+    return((POSIX::modf(shift))[0]);
 }
 
 sub _rfpart {
